#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <m17n.h>
#include <m17n-misc.h>

int
main (int argc, char **argv)
{
  char buf[256];
  MSymbol Mutf_8;
  int option = 0;
  int i;

  M17N_INIT ();
  Mutf_8 = msymbol ("utf-8");
  for (i = 1; i < argc; i++)
    {
      if (strcmp (argv[i], "--sp-cm") == 0)
	option |= MTEXT_LBO_SP_CM;
      else if (strcmp (argv[i], "--korean-sp") == 0)
	option |= MTEXT_LBO_KOREAN_SP;
      else if (strcmp (argv[i], "--ai-as-id") == 0)
	option |= MTEXT_LBO_AI_AS_ID;
      else
	{
	  printf ("Usage: linebreak [--sp-cm] [--korean-sp] [--ai-as-id]\n");
	  exit (0);
	}
    }

  while (fgets (buf, 256, stdin) != NULL)
    {
      int len = strlen (buf);
      MText *mt1, *mt2;
      int pos, last_pos = -1;

      if (buf[len - 1] == '\n')
	len--;

      mt1 = mconv_decode_buffer (msymbol ("utf-8"), (unsigned char *) buf, len);
      pos = mtext_len (mt1) - 1;
      while ((pos = mtext_character (mt1, pos, 0, '\\')) >= 0)
	{
	  if (mtext_ref_char (mt1, pos + 1) == 'n')
	    {
	      mtext_del (mt1, pos + 1, pos + 2);
	      mtext_set_char (mt1, pos, '\n');
	    }
	}
      pos = mtext_len (mt1);
      mt2 = mtext_dup (mt1);
      while (pos > 0)
	{
	  int break_pos = mtext_line_break (mt1, pos, option, NULL);

	  if (break_pos > pos)
	    break;
	  if (break_pos != last_pos)
	    {
	      mtext_ins_char (mt2, break_pos, '|', 1);
	      last_pos = break_pos;
	    }
	  pos = break_pos - 1;
	}

      pos = mtext_len (mt2) - 1;
      while ((pos = mtext_character (mt2, pos, 0, '\n')) >= 0)
	{
	  mtext_set_char (mt2, pos, '\\');
	  mtext_ins_char (mt2, pos + 1, 'n', 1);
	}

      mconv_encode_stream (Mutf_8, mt2, stdout);
      printf ("\n");
      m17n_object_unref (mt1);
      m17n_object_unref (mt2);
    }

  M17N_FINI ();
  exit (0);
}
